use strict;
use feature 'switch';
use Test::More tests => 5;
use IO::Socket::INET6;
use DE_EPAGES::Core::API::Object::UDPClient;
use DE_EPAGES::Core::API::Error qw( HandleError );
use DE_EPAGES::Core::API::PerlTools qw( GetIPAddress );

use constant RECEIVE_TIMEOUT => 1.0;
use constant RECEIVE_MAX_LENGTH => 1000;

my $Host = 'localhost';
my $Port = 5151; ## no critic qw(ProhibitMagicNumbers)
my $Debug = 0;
my $Mode = $ARGV[0];
given( $Mode ) {
    when( undef ) {
        # test mode

        my $pid = fork();
        die "fork() failed: $!" unless defined $pid;
        if ($pid) {
            eval {
                sleep 1; # wait for the server tor start
                Client(); # run test cases
                sleep 1; # wait for the server to shutdown
                my $wpid = wait;
                is( $wpid, $pid, 'server has finished' );
            };
            HandleError( OnError => sub {
                kill('TERM', $pid);
            });
        }
        else {
            Server();
        }
    }
    when( 'server' ) {
        Server();
    }
    when( 'client' ) {
        Client();
    }
}

sub Server {
    my $IP = GetIPAddress( $Host );
    die('No IP address') unless defined $IP;
    my $Family = $IP =~ /:/ ? AF_INET6 : AF_INET;

    my $Socket = IO::Socket::INET6->new(
        LocalHost => $Host,
        LocalPort => $Port,
        Proto => 'udp',
        Domain => $Family
    ) or die "socket: $@";

    print "Awaiting UDP messages on port $Port\n" if $Debug;

    my $Shutdown;
    my $Request;
    my $MAXLEN = 1024; ## no critic qw(ProhibitMagicNumbers)
    while ($Socket->recv($Request, $MAXLEN)) {
        my $ClientHost = $Socket->peerhost;
        print "Client $ClientHost said '$Request'\n" if $Debug;
        $Socket->send("$Request done");

        if( $Request eq 'Shutdown' ) {
            $Shutdown = 1;
            last;
        }
    }

    if( $Shutdown ) {
        print "shutting down\n" if $Debug;
    }
    else {
        die "recv: $!";
    }
}

sub Client {
    my $UDPClient = DE_EPAGES::Core::API::Object::UDPClient->new( $Host, $Port );
    $UDPClient->send('Hello1');
    $UDPClient->send('Hello2');
    is( 'Hello1 done', $UDPClient->receive(RECEIVE_TIMEOUT, RECEIVE_MAX_LENGTH), 'receive 1' );
    is( 'Hello2 done', $UDPClient->receive(RECEIVE_TIMEOUT, RECEIVE_MAX_LENGTH), 'receive 2' );
    is( 'Hello3 done', $UDPClient->sendReceive('Hello3'), 'sendReceive' );

    $UDPClient->send('Shutdown');
    is( 'Shutdown done', $UDPClient->receive(RECEIVE_TIMEOUT, RECEIVE_MAX_LENGTH), 'receive Shutdown' );
}
