#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::BuildLoginData
# state        public
#----------------------------------------------------------------------------------------
# description  Test for Login Data Builder (Live and Test)
#========================================================================================
package DE_EPAGES::DHLIntraship::t::BuildLoginData;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::DeveloperAuthentication;
use DE_EPAGES::DHLIntraship::API::BuildLoginData;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject::Extends;
use Test::Exception;
use Test::More;

sub Main {
    testToGetLoginDataForTestTransaction();
    testToGetLoginDataWithoutShippingFulfillmentType();
    testToGetLoginDataForLiveTransaction();

    done_testing();
}

sub testToGetLoginDataForTestTransaction {
    my $subTestName = (caller(0))[3];

    my $IsTest = 1;
    my $EndPoint = _GetBuildEndPointWithMockedLoadRootObject($IsTest);
    my $DeveloperAuthentication = _GetDeveloperAuthentication($IsTest);
    my $LoginData = DE_EPAGES::DHLIntraship::API::BuildLoginData->new()
        ->withEndPoint($EndPoint)
        ->withDeveloperAuthentication($DeveloperAuthentication)
    ;

    my $hResult = $LoginData->getAsHash();
    my $hExpectedLoginData ={
        'UserName' => 'abcTest',
        'Password' => 'defTest',
        'EndPointURL' => 'http://test.dummy.url.de',
        'Realm' => 'dummyRealm',
    };
    is_deeply($hResult, $hExpectedLoginData, "$subTestName - logindata for test Transaction");
}

sub testToGetLoginDataWithoutShippingFulfillmentType {
    my $subTestName = (caller(0))[3];
    my $IsTest = 1;
    my $EndPoint = _GetBuildEndPointWithMockedLoadRootObjectAndWithoutShippingFulfillmentTypes($IsTest);
    throws_ok {
    DE_EPAGES::DHLIntraship::API::BuildLoginData->new()
        ->withEndPoint($EndPoint);
    } qr/NoShippingFulfillmentType/,
              "$subTestName - test error handling if there isn't a ShippingFulfillmentTypes";
}

sub testToGetLoginDataForLiveTransaction {
    my $subTestName = (caller(0))[3];

    my $IsTest = 0;
    my $EndPoint = _GetBuildEndPointWithMockedLoadRootObject($IsTest);
    my $DeveloperAuthentication = _GetDeveloperAuthentication($IsTest);
    my $LoginData = DE_EPAGES::DHLIntraship::API::BuildLoginData->new()
        ->withEndPoint($EndPoint)
        ->withDeveloperAuthentication($DeveloperAuthentication)
    ;

    my $hResult = $LoginData->getAsHash();
    my $hExpectedLoginData ={
        'UserName' => 'abcLive',
        'Password' => 'defLive',
        'EndPointURL' => 'http://live.dummy.url.de',
        'Realm' => 'dummyRealm',
    };
    is_deeply($hResult, $hExpectedLoginData, "$subTestName - logindata for live Transaction");
}

sub _GetDeveloperAuthentication {
    my ($IsTest) = @_;

    my $DeveloperAuthentication = DE_EPAGES::DHLIntraship::API::DeveloperAuthentication->new($IsTest);
    my $MockedDeveloperAuthentication = Test::MockObject::Extends->new($DeveloperAuthentication);

    $MockedDeveloperAuthentication->mock('getUserName', sub {
        my $self = shift;
        if($self->{'IsTest'}) {
            return 'abcTest';
        } else {
            return 'abcLive';
        }
    });

    $MockedDeveloperAuthentication->mock('getPassWord', sub {
        my $self = shift;
        if($self->{'IsTest'}) {
            return 'defTest';
        } else {
            return 'defLive';
        }
    });

    $MockedDeveloperAuthentication->mock('getRealm', sub {
        return 'dummyRealm';
    });

    return $DeveloperAuthentication;
}

sub _GetBuildEndPointWithMockedLoadRootObject {
    my ($IsTest) = @_;

    my $EndPoint = DE_EPAGES::DHLIntraship::API::BuildEndPoint->new($IsTest);
    my $MockedEndPoint = Test::MockObject::Extends->new($EndPoint);
    $MockedEndPoint->mock('_LoadRootObject', sub {
        my $System = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', {'Alias'=>'Systeml'});
        $System->mock('folder', sub {
            my ($self, $FolderName) = @_;
            if($FolderName eq 'ShippingFulfillmentTypes') {
                return _GetShippingFulfillmentTypes(1);
            }
        });
        return $System;
    });

    return $MockedEndPoint;
}

sub _GetBuildEndPointWithMockedLoadRootObjectAndWithoutShippingFulfillmentTypes {
    my ($IsTest) = @_;

    my $EndPoint = DE_EPAGES::DHLIntraship::API::BuildEndPoint->new($IsTest);
    my $MockedEndPoint = Test::MockObject::Extends->new($EndPoint);
    $MockedEndPoint->mock('_LoadRootObject', sub {
        my $System = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', {'Alias'=>'Systeml'});
        $System->mock('folder', sub {
            my ($self, $FolderName) = @_;
            if($FolderName eq 'ShippingFulfillmentTypes') {
                return _GetShippingFulfillmentTypes(0);
            }
        });
        return $System;
    });

    return $MockedEndPoint;
}

sub _GetShippingFulfillmentTypes {
    my ($ExistsChild) = @_;
    my $ShippingFulfillmentTypes = Test::MockObject->new();
    $ShippingFulfillmentTypes->mock( 'existsChild', sub { return $ExistsChild; } );
    $ShippingFulfillmentTypes->mock( 'child', sub { return _GetShippingFulfillemntType(); } );
    return $ShippingFulfillmentTypes;
}

sub _GetShippingFulfillemntType {
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',{
        'WebserviceURL' => 'http://live.dummy.url.de',
        'WebserviceTestURL' => 'http://test.dummy.url.de',
    });
}

RunScript( Sub => \&Main );