#========================================================================================
# package      DE_EPAGES::WebInterface::t::HttpMethod
# state        private
#========================================================================================
package DE_EPAGES::WebInterface::t::HttpMethod;
use base DE_EPAGES::Core::API::Test;
use Test::More tests => 20;
use IO::String;
use HTTP::Request;
use LWP::UserAgent;
use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );
use DE_EPAGES::Core::API::File qw( ExistsFile MoveFile );
use DE_EPAGES::Core::API::IniConfig;
use strict;

my $SERVLET_CONFIG = $ENV{'EPAGES_CONFIG'}.'/Servlet.conf';
my $SERVLET_CONFIG_BAK = $ENV{'EPAGES_CONFIG'}.'/Servlet.conf.bak';

sub init {

    die "backup file $SERVLET_CONFIG_BAK still exists" if ExistsFile( $SERVLET_CONFIG_BAK );

    MoveFile( $SERVLET_CONFIG, $SERVLET_CONFIG_BAK, { 'stat' => 'preserve' } );

    my $Config = DE_EPAGES::Core::API::IniConfig->create( FileName => $SERVLET_CONFIG, CaseSensitive => 1 );
    $Config->set( 'Servlets', '0', 'Echo' );
    $Config->set( 'Echo', 'expression', '1' );
    $Config->set( 'Echo', 'package', 'DE_EPAGES::WebInterface::Servlet::Echo' );
    $Config->rewrite;

    sleep 2;
}

sub ParseContent {
    my ($Echo) = @_;

    my $Stream = IO::String->new($Echo);
    my $Line = $Stream->getline;
    my $Request;
    if( $Line =~ /^(\w+)\s*(\S+)/ ) {
        $Request = HTTP::Request->new( $1, $2 );
    }
    else {
        die "Missing request method in '$Echo'";
    }

    while( $_ = $Stream->getline ) {
        chomp;
        last if $_ eq '';

        if( /^(.*?)\s*:\s*(.*)/ ) {
            $Request->push_header( $1, $2 );
        }
        else {
            warn "invalid header line: $_";
        }
    }

    my $ContentLength = $Request->content_length // 0;
    if( $ContentLength ) {
        my $Content;
        $Stream->read( $Content, $ContentLength );
        $Request->content( $Content );
    }

    return $Request;
}

sub EchoRequest {
    my ($Request) = @_;

    my $UserAgent = LWP::UserAgent->new;
    my $Response = $UserAgent->request( $Request );
#    print $Response->as_string;

    return ParseContent( $Response->content );
}

sub cleanup {

    if( ExistsFile( $SERVLET_CONFIG_BAK ) ) {
        MoveFile( $SERVLET_CONFIG_BAK, $SERVLET_CONFIG, { 'stat' => 'preserve' } );
    }
}

sub test {

    my $DomainName = EpagesConfig()->get('DE_EPAGES::Object', 'SystemDomainName');
    my $TestURL = "http://$DomainName/epages/TestRequest.sf";
    note "using URL: $TestURL";

    my $Echo = EchoRequest( HTTP::Request->new('GET', $TestURL) );
    is( $Echo->method, 'GET', 'GET: method' );
    is( $Echo->uri, '/epages/TestRequest.sf', 'GET: uri' );
    is( $Echo->content_length, 0, 'GET: content_length' );
    is( $Echo->content, '', 'GET: content' );

    my $PostContent = 'Name1=Value1&Name2=Value2';
    my $PostRequest = HTTP::Request->new('POST', $TestURL);
    $PostRequest->content_type('application/x-www-form-urlencoded');
    $PostRequest->content($PostContent);

    $Echo = EchoRequest( $PostRequest );
    is( $Echo->method, 'POST', 'POST: method' );
    is( $Echo->uri, '/epages/TestRequest.sf', 'POST: uri' );
    is( $Echo->content_length, length $PostContent, 'POST: content_length' );
    is( $Echo->content, $PostContent, 'POST: content' );

    my $PutContent = '<tag>Put Content</tag>';
    my $PutRequest = HTTP::Request->new('PUT', $TestURL);
    $PutRequest->content_type('text/xml');
    $PutRequest->content($PutContent);

    $Echo = EchoRequest( $PutRequest );
    is( $Echo->method, 'PUT', 'PUT: method' );
    is( $Echo->uri, '/epages/TestRequest.sf', 'PUT: uri' );
    is( $Echo->content_length, length $PutContent, 'PUT: content_length' );
    is( $Echo->content, $PutContent, 'PUT: content' );

    $Echo = EchoRequest( HTTP::Request->new('DELETE', $TestURL) );
    is( $Echo->method, 'DELETE', 'DELETE: method' );
    is( $Echo->uri, '/epages/TestRequest.sf', 'DELETE: uri' );
    is( $Echo->content_length, 0, 'DELETE: content_length' );
    is( $Echo->content, '', 'DELETE: content' );

    # disabled for now because we don't know if the WebServer is configured to handle HTTP method 'Dummy'
    #$Echo = EchoRequest( HTTP::Request->new('Dummy', $TestURL) );
    #is( $Echo->method, 'Dummy', 'Dummy: method' );
    #is( $Echo->uri, '/epages/TestRequest.sf', 'Dummy: uri' );

    $TestURL = 'http://' . uc($DomainName) . '/epages/TestRequest.sf';
    $Echo = EchoRequest( HTTP::Request->new('GET', $TestURL) );
    is( $Echo->method, 'GET', 'UPPERCASE_DOMAIN GET: method' );
    is( $Echo->uri, '/epages/TestRequest.sf', 'UPPERCASE_DOMAIN GET: uri' );
    is( $Echo->content_length, 0, 'UPPERCASE_DOMAIN GET: content_length' );
    is( $Echo->content, '', 'UPPERCASE_DOMAIN GET: content' );

    return;
}

__PACKAGE__->runScript;
